@@ -155,6 +155,36 @@ ephy_history_service_dispose (GObject *object)
   G_OBJECT_CLASS (ephy_history_service_parent_class)->dispose (object);
 }
 
+static void
+ephy_history_service_constructed (GObject *object)
+{
+  EphyHistoryService *self = EPHY_HISTORY_SERVICE (object);
+
+  G_OBJECT_CLASS (ephy_history_service_parent_class)->constructed (object);
+
+  self->queue = g_async_queue_new ();
+
+  /* This value is checked in several functions to verify that they are only
+   * ever run on the history thread. Accordingly, we'd better be sure it's set
+   * before it is checked for the first time. That requires a lock here. */
+  g_mutex_lock (&self->history_thread_mutex);
+  self->history_thread = g_thread_new ("EphyHistoryService", (GThreadFunc)run_history_service_thread, self);
+
+  /* Additionally, make sure the SQLite connection has really been opened before
+   * returning. We need this so that we can test that using a read-only service
+   * at the same time as a read/write service does not cause the read/write
+   * service to break. This delay is required because we need to be sure the
+   * read/write service has completed initialization before attempting to open
+   * the read-only service, or initializing the read-only service will fail.
+   * This isn't needed except in test mode, because only tests might run
+   * multiple history services, but it's harmless and cleaner to do always.
+   */
+  while (!self->history_thread_initialized)
+    g_cond_wait (&self->history_thread_initialized_condition, &self->history_thread_mutex);
+
+  g_mutex_unlock (&self->history_thread_mutex);
+}
+
 static gboolean
 emit_urls_visited (EphyHistoryService *self)
 {
@@ -181,6 +211,7 @@ ephy_history_service_class_init (EphyHistoryServiceClass *klass)
 
   gobject_class->finalize = ephy_history_service_finalize;
   gobject_class->dispose = ephy_history_service_dispose;
+  gobject_class->constructed = ephy_history_service_constructed;
   gobject_class->get_property = ephy_history_service_get_property;
   gobject_class->set_property = ephy_history_service_set_property;
 
@@ -253,7 +284,7 @@ ephy_history_service_class_init (EphyHistoryServiceClass *klass)
                          "History filename",
                          "The filename of the SQLite file holding containing history",
                          NULL,
-                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_STRINGS);
+                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
 
   obj_properties[PROP_READ_ONLY] =
     g_param_spec_boolean ("read-only",
@@ -268,17 +299,15 @@ ephy_history_service_class_init (EphyHistoryServiceClass *klass)
 static void
 ephy_history_service_init (EphyHistoryService *self)
 {
-  self->history_thread = g_thread_new ("EphyHistoryService", (GThreadFunc)run_history_service_thread, self);
-  self->queue = g_async_queue_new ();
 }
 
 EphyHistoryService *
-ephy_history_service_new (const char *history_filename,
-                          gboolean    read_only)
+ephy_history_service_new (const char               *history_filename,
+                          EphySQLiteConnectionMode  mode)
 {
   return EPHY_HISTORY_SERVICE (g_object_new (EPHY_TYPE_HISTORY_SERVICE,
                                              "history-filename", history_filename,
-                                             "read-only", read_only,
+                                             "read-only", mode == EPHY_SQLITE_CONNECTION_MODE_READ_ONLY,
                                              NULL));
 }
 
@@ -329,29 +358,37 @@ ephy_history_service_send_message (EphyHistoryService *self, EphyHistoryServiceM
 }
 
 static void
-ephy_history_service_commit (EphyHistoryService *self)
+ephy_history_service_open_transaction (EphyHistoryService *self)
 {
   GError *error = NULL;
   g_assert (self->history_thread == g_thread_self ());
 
-  if (NULL == self->history_database)
+  if (self->history_database == NULL ||
+      self->read_only)
     return;
 
-  if (self->read_only)
+  ephy_sqlite_connection_begin_transaction (self->history_database, &error);
+  if (error != NULL) {
+    g_error ("Could not open history database transaction: %s", error->message);
+    g_error_free (error);
+  }
+}
+
+static void
+ephy_history_service_commit_transaction (EphyHistoryService *self)
+{
+  GError *error = NULL;
+  g_assert (self->history_thread == g_thread_self ());
+
+  if (self->history_database == NULL ||
+      self->read_only)
     return;
 
   ephy_sqlite_connection_commit_transaction (self->history_database, &error);
-  if (NULL != error) {
-    g_warning ("Could not commit idle history database transaction: %s", error->message);
+  if (error != NULL) {
+    g_warning ("Could not commit history database transaction: %s", error->message);
     g_error_free (error);
   }
-  ephy_sqlite_connection_begin_transaction (self->history_database, &error);
-  if (NULL != error) {
-    g_warning ("Could not start long-running history database transaction: %s", error->message);
-    g_error_free (error);
-  }
-
-  self->scheduled_to_commit = FALSE;
 }
 
 static void
@@ -359,7 +396,7 @@ ephy_history_service_enable_foreign_keys (EphyHistoryService *self)
 {
   GError *error = NULL;
 
-  if (NULL == self->history_database)
+  if (self->history_database == NULL)
     return;
 
   ephy_sqlite_connection_execute (self->history_database,
@@ -378,31 +415,33 @@ ephy_history_service_open_database_connections (EphyHistoryService *self)
 
   g_assert (self->history_thread == g_thread_self ());
 
-  self->history_database = ephy_sqlite_connection_new ();
+  if (self->history_database != NULL)
+    g_object_unref (self->history_database);
+
+  self->history_database = ephy_sqlite_connection_new (self->read_only ? EPHY_SQLITE_CONNECTION_MODE_READ_ONLY
+                                                                       : EPHY_SQLITE_CONNECTION_MODE_READWRITE);
   ephy_sqlite_connection_open (self->history_database, self->history_filename, &error);
   if (error) {
     g_object_unref (self->history_database);
     self->history_database = NULL;
-    g_warning ("Could not open history database at %s: %s", self->history_filename, error->message);
+
+    /* Opening the database is expected to fail if it's being opened in read-
+     * only mode and does not already exist. Otherwise, this is bad. */
+    if (!self->read_only ||
+        !g_error_matches (error, EPHY_SQLITE_ERROR, SQLITE_CANTOPEN) ||
+        g_file_test (self->history_filename, G_FILE_TEST_EXISTS)) {
+      g_warning ("Could not open history database at %s: %s", self->history_filename, error->message);
+    }
     g_error_free (error);
     return FALSE;
   }
 
   ephy_history_service_enable_foreign_keys (self);
 
-  ephy_sqlite_connection_begin_transaction (self->history_database, &error);
-  if (error) {
-    g_warning ("Could not begin long running transaction in history database: %s", error->message);
-    g_error_free (error);
-    return FALSE;
-  }
-
-  if (ephy_history_service_initialize_hosts_table (self) == FALSE ||
-      ephy_history_service_initialize_urls_table (self) == FALSE ||
-      ephy_history_service_initialize_visits_table (self) == FALSE)
-    return FALSE;
-
-  return TRUE;
+  return self->read_only ||
+          (ephy_history_service_initialize_hosts_table (self) &&
+           ephy_history_service_initialize_urls_table (self) &&
+           ephy_history_service_initialize_visits_table (self));
 }
 
 static void
@@ -415,57 +454,11 @@ ephy_history_service_close_database_connections (EphyHistoryService *self)
   self->history_database = NULL;
 }
 
-static void
-ephy_history_service_clear_all (EphyHistoryService *self)
-{
-  char *journal_filename;
-
-  if (self->history_database == NULL)
-    return;
-
-  if (self->read_only)
-    return;
-
-  ephy_sqlite_connection_close (self->history_database);
-
-  if (g_unlink (self->history_filename) == -1)
-    g_warning ("Failed to delete %s: %s", self->history_filename, g_strerror (errno));
-
-  journal_filename = g_strdup_printf ("%s-journal", self->history_filename);
-  if (g_unlink (journal_filename) == -1 && errno != ENOENT)
-    g_warning ("Failed to delete %s: %s", journal_filename, g_strerror (errno));
-  g_free (journal_filename);
-
-  ephy_history_service_open_database_connections (self);
-}
-
-static gboolean
-ephy_history_service_is_scheduled_to_quit (EphyHistoryService *self)
-{
-  return self->scheduled_to_quit;
-}
-
-static gboolean
-ephy_history_service_is_scheduled_to_commit (EphyHistoryService *self)
-{
-  return self->scheduled_to_commit;
-}
-
-void
-ephy_history_service_schedule_commit (EphyHistoryService *self)
-{
-  if (!self->read_only)
-    self->scheduled_to_commit = TRUE;
-}
-
 static gboolean
 ephy_history_service_execute_quit (EphyHistoryService *self, gpointer data, gpointer *result)
 {
   g_assert (self->history_thread == g_thread_self ());
 
-  if (ephy_history_service_is_scheduled_to_commit (self))
-    ephy_history_service_commit (self);
-
   g_async_queue_unref (self->queue);
 
   self->scheduled_to_quit = TRUE;
@@ -477,26 +470,35 @@ static gpointer
 run_history_service_thread (EphyHistoryService *self)
 {
   EphyHistoryServiceMessage *message;
+  gboolean success;
 
+  /* Note that self->history_thread is only written once, and that's guaranteed
+   * to have occurred before we enter this critical section due to this mutex.
+   * Accordingly, we do not need to use the mutex when performing these
+   * assertions in other functions.
+   */
+  g_mutex_lock (&self->history_thread_mutex);
   g_assert (self->history_thread == g_thread_self ());
 
-  if (ephy_history_service_open_database_connections (self) == FALSE)
+  success = ephy_history_service_open_database_connections (self);
+
+  self->history_thread_initialized = TRUE;
+  g_cond_signal (&self->history_thread_initialized_condition);
+  g_mutex_unlock (&self->history_thread_mutex);
+
+  if (!success)
     return NULL;
 
   do {
     message = g_async_queue_try_pop (self->queue);
     if (!message) {
-      /* Schedule commit if needed. */
-      if (ephy_history_service_is_scheduled_to_commit (self))
-        ephy_history_service_commit (self);
-
       /* Block the thread until there's data in the queue. */
       message = g_async_queue_pop (self->queue);
     }
 
     /* Process item. */
     ephy_history_service_process_message (self, message);
-  } while (!ephy_history_service_is_scheduled_to_quit (self));
+  } while (!self->scheduled_to_quit);
 
   ephy_history_service_close_database_connections (self);
 
@@ -574,7 +576,7 @@ ephy_history_service_execute_add_visit_helper (EphyHistoryService *self, EphyHis
   ephy_history_service_update_host_row (self, visit->url->host);
 
   /* A NULL return here means that the URL does not yet exist in the database */
-  if (NULL == ephy_history_service_get_url_row (self, visit->url->url, visit->url)) {
+  if (ephy_history_service_get_url_row (self, visit->url->url, visit->url) == NULL) {
     visit->url->last_visit_time = visit->visit_time;
     visit->url->visit_count = 1;
 
@@ -624,8 +626,6 @@ ephy_history_service_execute_add_visits (EphyHistoryService *self, GList *visits
     visits = visits->next;
   }
 
-  ephy_history_service_schedule_commit (self);
-
   return success;
 }
 
@@ -638,7 +638,7 @@ ephy_history_service_execute_find_visits (EphyHistoryService *self, EphyHistoryQ
   /* FIXME: We don't have a good way to tell the difference between failures and empty returns */
   while (current) {
     EphyHistoryPageVisit *visit = (EphyHistoryPageVisit *)current->data;
-    if (NULL == ephy_history_service_get_url_row (self, NULL, visit->url)) {
+    if (ephy_history_service_get_url_row (self, NULL, visit->url) == NULL) {
       ephy_history_page_visit_list_free (visits);
       g_warning ("Tried to process an orphaned page visit");
       return FALSE;
@@ -811,7 +811,7 @@ ephy_history_service_execute_set_url_title (EphyHistoryService *self,
   if (self->read_only)
     return FALSE;
 
-  if (NULL == ephy_history_service_get_url_row (self, NULL, url)) {
+  if (ephy_history_service_get_url_row (self, NULL, url) == NULL) {
     /* The URL is not yet in the database, so we can't update it.. */
     g_free (title);
     return FALSE;
@@ -821,7 +821,6 @@ ephy_history_service_execute_set_url_title (EphyHistoryService *self,
     g_free (url->title);
     url->title = title;
     ephy_history_service_update_url_row (self, url);
-    ephy_history_service_schedule_commit (self);
 
     ctx = signal_emission_context_new (self,
                                        ephy_history_url_copy (url),
@@ -877,7 +876,6 @@ ephy_history_service_execute_set_url_zoom_level (EphyHistoryService *self,
 
   host->zoom_level = zoom_level;
   ephy_history_service_update_host_row (self, host);
-  ephy_history_service_schedule_commit (self);
 
   return TRUE;
 }
@@ -916,13 +914,12 @@ ephy_history_service_execute_set_url_hidden (EphyHistoryService *self,
 
   hidden = url->hidden;
 
-  if (NULL == ephy_history_service_get_url_row (self, NULL, url)) {
+  if (ephy_history_service_get_url_row (self, NULL, url) == NULL) {
     /* The URL is not yet in the database, so we can't update it.. */
     return FALSE;
   } else {
     url->hidden = hidden;
     ephy_history_service_update_url_row (self, url);
-    ephy_history_service_schedule_commit (self);
     return TRUE;
   }
 }
@@ -962,12 +959,11 @@ ephy_history_service_execute_set_url_thumbnail_time (EphyHistoryService *self,
 
   thumbnail_time = url->thumbnail_time;
 
-  if (NULL == ephy_history_service_get_url_row (self, NULL, url))
+  if (ephy_history_service_get_url_row (self, NULL, url) == NULL)
     return FALSE;
   else {
     url->thumbnail_time = thumbnail_time;
     ephy_history_service_update_url_row (self, url);
-    ephy_history_service_schedule_commit (self);
     return TRUE;
   }
 }
@@ -1094,7 +1090,6 @@ ephy_history_service_execute_delete_urls (EphyHistoryService *self,
   }
 
   ephy_history_service_delete_orphan_hosts (self);
-  ephy_history_service_schedule_commit (self);
 
   return TRUE;
 }
@@ -1121,7 +1116,6 @@ ephy_history_service_execute_delete_host (EphyHistoryService    *self,
     return FALSE;
 
   ephy_history_service_delete_host_row (self, host);
-  ephy_history_service_schedule_commit (self);
 
   ctx = signal_emission_context_new (self, g_strdup (host->url),
                                      (GDestroyNotify)g_free);
@@ -1138,11 +1132,25 @@ ephy_history_service_execute_clear (EphyHistoryService *self,
                                     gpointer            pointer,
                                     gpointer           *result)
 {
-  if (self->read_only)
+  char *journal_filename;
+
+  if (self->history_database == NULL ||
+      self->read_only)
     return FALSE;
 
-  ephy_history_service_clear_all (self);
-  ephy_history_service_schedule_commit (self);
+  ephy_history_service_commit_transaction (self);
+  ephy_sqlite_connection_close (self->history_database);
+
+  if (g_unlink (self->history_filename) == -1)
+    g_warning ("Failed to delete %s: %s", self->history_filename, g_strerror (errno));
+
+  journal_filename = g_strdup_printf ("%s-journal", self->history_filename);
+  if (g_unlink (journal_filename) == -1 && errno != ENOENT)
+    g_warning ("Failed to delete %s: %s", journal_filename, g_strerror (errno));
+  g_free (journal_filename);
+
+  ephy_history_service_open_database_connections (self);
+  ephy_history_service_open_transaction (self);
 
   return TRUE;
 }
@@ -1248,10 +1256,13 @@ ephy_history_service_process_message (EphyHistoryService        *self,
 
   method = methods[message->type];
   message->result = NULL;
-  if (message->service->history_database)
+  if (message->service->history_database) {
+    ephy_history_service_open_transaction (self);
     message->success = method (message->service, message->method_argument, &message->result);
-  else
+    ephy_history_service_commit_transaction (self);
+  } else {
     message->success = FALSE;
+  }
 
   if (message->callback || message->type == CLEAR)
     g_idle_add ((GSourceFunc)ephy_history_service_execute_job_callback, message);
