@@ -27,6 +27,8 @@
 #include "ephy-debug.h"
 #include "ephy-file-helpers.h"
 
+#include <string.h>
+
 #define EPHY_BOOKMARKS_FILE "bookmarks.gvdb"
 
 struct _EphyBookmarksManager {
@@ -38,7 +40,10 @@ struct _EphyBookmarksManager {
   gchar      *gvdb_filename;
 };
 
-G_DEFINE_TYPE (EphyBookmarksManager, ephy_bookmarks_manager, G_TYPE_OBJECT)
+static void list_model_iface_init     (GListModelInterface *iface);
+
+G_DEFINE_TYPE_EXTENDED (EphyBookmarksManager, ephy_bookmarks_manager, G_TYPE_OBJECT, 0,
+                        G_IMPLEMENT_INTERFACE (G_TYPE_LIST_MODEL, list_model_iface_init))
 
 enum {
   BOOKMARK_ADDED,
@@ -156,7 +161,8 @@ ephy_bookmarks_manager_class_init (EphyBookmarksManagerClass *klass)
                   G_SIGNAL_RUN_LAST,
                   0,
                   NULL, NULL, NULL,
-                  G_TYPE_NONE, 1,
+                  G_TYPE_NONE, 2,
+                  G_TYPE_STRING,
                   G_TYPE_INT);
 }
 
@@ -171,7 +177,7 @@ ephy_bookmarks_manager_init (EphyBookmarksManager *self)
   self->tags = g_sequence_new (g_free);
 
   g_sequence_insert_sorted (self->tags,
-                            g_strdup ("Favorites"),
+                            g_strdup (EPHY_BOOKMARKS_FAVORITES_TAG),
                             (GCompareDataFunc)ephy_bookmark_tags_compare,
                             NULL);
 
@@ -182,6 +188,40 @@ ephy_bookmarks_manager_init (EphyBookmarksManager *self)
   ephy_bookmarks_manager_load_from_file (self);
 }
 
+static GType
+ephy_bookmarks_manager_list_model_get_item_type (GListModel *model)
+{
+  return EPHY_TYPE_BOOKMARK;
+}
+
+static guint
+ephy_bookmarks_manager_list_model_get_n_items (GListModel *model)
+{
+  EphyBookmarksManager *self = EPHY_BOOKMARKS_MANAGER (model);
+
+  return g_sequence_get_length (self->bookmarks);
+}
+
+static gpointer
+ephy_bookmarks_manager_list_model_get_item (GListModel *model,
+                                            guint       position)
+{
+  EphyBookmarksManager *self = EPHY_BOOKMARKS_MANAGER (model);
+  GSequenceIter *iter;
+
+  iter = g_sequence_get_iter_at_pos (self->bookmarks, position);
+
+  return g_object_ref (g_sequence_get (iter));
+}
+
+static void
+list_model_iface_init (GListModelInterface *iface)
+{
+  iface->get_item_type = ephy_bookmarks_manager_list_model_get_item_type;
+  iface->get_n_items = ephy_bookmarks_manager_list_model_get_n_items;
+  iface->get_item = ephy_bookmarks_manager_list_model_get_item;
+}
+
 static void
 bookmark_title_changed_cb (EphyBookmark         *bookmark,
                            GParamSpec           *pspec,
@@ -235,12 +275,23 @@ ephy_bookmarks_manager_watch_bookmark (EphyBookmarksManager *self,
                            G_CALLBACK (bookmark_tag_removed_cb), self, 0);
 }
 
+static void
+ephy_bookmarks_manager_unwatch_bookmark (EphyBookmarksManager *self,
+                                         EphyBookmark         *bookmark)
+{
+  g_signal_handlers_disconnect_by_func (bookmark, bookmark_title_changed_cb, self);
+  g_signal_handlers_disconnect_by_func (bookmark, bookmark_url_changed_cb, self);
+  g_signal_handlers_disconnect_by_func (bookmark, bookmark_tag_added_cb, self);
+  g_signal_handlers_disconnect_by_func (bookmark, bookmark_tag_removed_cb, self);
+}
+
 void
 ephy_bookmarks_manager_add_bookmark (EphyBookmarksManager *self,
                                      EphyBookmark         *bookmark)
 {
   GSequenceIter *iter;
   GSequenceIter *prev_iter;
+  gint position;
 
   g_return_if_fail (EPHY_IS_BOOKMARKS_MANAGER (self));
   g_return_if_fail (EPHY_IS_BOOKMARK (bookmark));
@@ -254,6 +305,11 @@ ephy_bookmarks_manager_add_bookmark (EphyBookmarksManager *self,
   if (g_sequence_iter_is_end (prev_iter)
       || ephy_bookmark_get_time_added (g_sequence_get (prev_iter)) != ephy_bookmark_get_time_added (bookmark)) {
     g_sequence_insert_before (iter, bookmark);
+
+    /* Update list */
+    position = g_sequence_iter_get_position (iter);
+    g_list_model_items_changed (G_LIST_MODEL (self), position - 1, 0, 1);
+
     g_signal_emit (self, signals[BOOKMARK_ADDED], 0, bookmark);
 
     ephy_bookmarks_manager_save_to_file_async (self, NULL,
@@ -301,6 +357,7 @@ ephy_bookmarks_manager_remove_bookmark (EphyBookmarksManager *self,
                                         EphyBookmark         *bookmark)
 {
   GSequenceIter *iter;
+  gint position;
 
   g_return_if_fail (EPHY_IS_BOOKMARKS_MANAGER (self));
   g_return_if_fail (EPHY_IS_BOOKMARK (bookmark));
@@ -312,24 +369,24 @@ ephy_bookmarks_manager_remove_bookmark (EphyBookmarksManager *self,
                    ephy_bookmark_get_url (bookmark)) == 0)
       break;
   }
+  g_assert (!g_sequence_iter_is_end (iter));
 
   /* Ensure the bookmark is removed from our list before the signal is emitted,
    * because this is the bookmark REMOVED signal after all, so callers expect
    * it to be already gone.
    */
   g_object_ref (bookmark);
+  position = g_sequence_iter_get_position (iter);
   g_sequence_remove (iter);
+  g_list_model_items_changed (G_LIST_MODEL (self), position, 1, 0);
   g_signal_emit (self, signals[BOOKMARK_REMOVED], 0, bookmark);
-  g_object_unref (bookmark);
 
   ephy_bookmarks_manager_save_to_file_async (self, NULL,
                                              (GAsyncReadyCallback)ephy_bookmarks_manager_save_to_file_warn_on_error_cb,
                                              NULL);
 
-  g_signal_handlers_disconnect_by_func (bookmark, bookmark_title_changed_cb, self);
-  g_signal_handlers_disconnect_by_func (bookmark, bookmark_url_changed_cb, self);
-  g_signal_handlers_disconnect_by_func (bookmark, bookmark_tag_added_cb, self);
-  g_signal_handlers_disconnect_by_func (bookmark, bookmark_tag_removed_cb, self);
+  ephy_bookmarks_manager_unwatch_bookmark (self, bookmark);
+  g_object_unref (bookmark);
 }
 
 EphyBookmark *
@@ -405,6 +462,9 @@ ephy_bookmarks_manager_delete_tag (EphyBookmarksManager *self, const char *tag)
   g_return_if_fail (EPHY_IS_BOOKMARKS_MANAGER (self));
   g_return_if_fail (tag != NULL);
 
+  if (strcmp (tag, EPHY_BOOKMARKS_FAVORITES_TAG) == 0)
+    return;
+
   iter = g_sequence_lookup (self->tags,
                             (gpointer)tag,
                             (GCompareDataFunc)ephy_bookmark_tags_compare,
@@ -418,7 +478,7 @@ ephy_bookmarks_manager_delete_tag (EphyBookmarksManager *self, const char *tag)
   /* Also remove the tag from each bookmark if they have it */
   g_sequence_foreach (self->bookmarks, (GFunc)ephy_bookmark_remove_tag, (gpointer)tag);
 
-  g_signal_emit (self, signals[TAG_DELETED], 0, position);
+  g_signal_emit (self, signals[TAG_DELETED], 0, tag, position);
 }
 
 gboolean
@@ -486,39 +546,6 @@ ephy_bookmarks_manager_get_bookmarks_with_tag (EphyBookmarksManager *self,
   return bookmarks;
 }
 
-static int
-compare_smart_bookmarks (EphyBookmark *a,
-                         EphyBookmark *b)
-{
-  return g_utf8_collate (ephy_bookmark_get_title (a), ephy_bookmark_get_title (b));
-}
-
-GSequence *
-ephy_bookmarks_manager_get_smart_bookmarks (EphyBookmarksManager *self)
-{
-  GSequence *bookmarks;
-  GSequenceIter *iter;
-
-  g_return_val_if_fail (EPHY_IS_BOOKMARKS_MANAGER (self), NULL);
-
-  bookmarks = g_sequence_new (g_object_unref);
-
-  for (iter = g_sequence_get_begin_iter (self->bookmarks);
-       !g_sequence_iter_is_end (iter);
-       iter = g_sequence_iter_next (iter)) {
-    EphyBookmark *bookmark = g_sequence_get (iter);
-
-    if (ephy_bookmark_is_smart (bookmark)) {
-      g_sequence_insert_sorted (bookmarks,
-                                g_object_ref (bookmark),
-                                (GCompareDataFunc)compare_smart_bookmarks,
-                                NULL);
-    }
-  }
-
-  return bookmarks;
-}
-
 GSequence *
 ephy_bookmarks_manager_get_tags (EphyBookmarksManager *self)
 {
