@@ -28,12 +28,12 @@
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-shell.h"
 #include "ephy-embed-utils.h"
-#include "ephy-encodings.h"
 #include "ephy-file-chooser.h"
 #include "ephy-file-helpers.h"
 #include "ephy-gui.h"
 #include "ephy-langs.h"
 #include "ephy-prefs.h"
+#include "ephy-search-engine-dialog.h"
 #include "ephy-session.h"
 #include "ephy-settings.h"
 #include "ephy-shell.h"
@@ -77,7 +77,6 @@ struct _PrefsDialog {
   GtkWidget *download_button_label;
   GtkWidget *automatic_downloads_checkbutton;
   GtkWidget *search_box;
-  GtkWidget *search_engine_combo;
   GtkWidget *session_box;
   GtkWidget *restore_session_checkbutton;
   GtkWidget *popups_allow_checkbutton;
@@ -102,7 +101,6 @@ struct _PrefsDialog {
   GtkWidget *clear_personal_data_button;
 
   /* language */
-  GtkWidget *default_encoding_combo;
   GtkTreeView *lang_treeview;
   GtkWidget *lang_add_button;
   GtkWidget *lang_remove_button;
@@ -155,13 +153,6 @@ enum {
   NUM_COLS
 };
 
-enum {
-  SEARCH_ENGINE_COL_NAME,
-  SEARCH_ENGINE_COL_STOCK_URL,
-  SEARCH_ENGINE_COL_URL,
-  SEARCH_ENGINE_NUM_COLS
-};
-
 G_DEFINE_TYPE (PrefsDialog, prefs_dialog, GTK_TYPE_DIALOG)
 
 #ifdef ENABLE_SYNC
@@ -583,6 +574,19 @@ on_manage_passwords_button_clicked (GtkWidget   *button,
   gtk_window_present (GTK_WINDOW (passwords_dialog));
 }
 
+static void
+on_search_engine_dialog_button_clicked (GtkWidget   *button,
+                                        PrefsDialog *dialog)
+{
+  GtkWindow *search_engine_dialog;
+
+  search_engine_dialog = GTK_WINDOW (ephy_search_engine_dialog_new ());
+
+  gtk_window_set_transient_for (search_engine_dialog, GTK_WINDOW (dialog));
+  gtk_window_set_modal (search_engine_dialog, TRUE);
+  gtk_window_present (search_engine_dialog);
+}
+
 static void
 prefs_dialog_class_init (PrefsDialogClass *klass)
 {
@@ -601,7 +605,6 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, custom_homepage_entry);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, automatic_downloads_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, search_box);
-  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, search_engine_combo);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, session_box);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, restore_session_checkbutton);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, popups_allow_checkbutton);
@@ -628,7 +631,6 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, clear_personal_data_button);
 
   /* language */
-  gtk_widget_class_bind_template_child (widget_class, PrefsDialog, default_encoding_combo);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, lang_treeview);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, lang_add_button);
   gtk_widget_class_bind_template_child (widget_class, PrefsDialog, lang_remove_button);
@@ -649,6 +651,7 @@ prefs_dialog_class_init (PrefsDialogClass *klass)
 
   gtk_widget_class_bind_template_callback (widget_class, on_manage_cookies_button_clicked);
   gtk_widget_class_bind_template_callback (widget_class, on_manage_passwords_button_clicked);
+  gtk_widget_class_bind_template_callback (widget_class, on_search_engine_dialog_button_clicked);
 }
 
 static void
@@ -666,128 +669,6 @@ css_edit_button_clicked_cb (GtkWidget   *button,
   g_object_unref (css_file);
 }
 
-static gboolean
-combo_get_mapping (GValue   *value,
-                   GVariant *variant,
-                   gpointer  user_data)
-{
-  GtkTreeModel *model;
-  GtkTreeIter iter;
-  gboolean valid = FALSE;
-  const char *settings_name;
-  int i;
-
-  model = gtk_combo_box_get_model (GTK_COMBO_BOX (user_data));
-  valid = gtk_tree_model_get_iter_first (model, &iter);
-  settings_name = g_variant_get_string (variant, NULL);
-  i = 0;
-
-  while (valid) {
-    char *item_name;
-    gtk_tree_model_get (model, &iter, 1, &item_name, -1);
-
-    if (g_strcmp0 (item_name, settings_name) == 0) {
-      g_value_set_int (value, i);
-      g_free (item_name);
-      break;
-    }
-
-    i++;
-    valid = gtk_tree_model_iter_next (model, &iter);
-    g_free (item_name);
-  }
-
-  return TRUE;
-}
-
-static GVariant *
-combo_set_mapping (const GValue       *value,
-                   const GVariantType *expected_type,
-                   gpointer            user_data)
-{
-  GVariant *variant = NULL;
-  GtkTreeModel *model;
-  GtkTreeIter iter;
-  gboolean valid = FALSE;
-  int n;
-
-  n = g_value_get_int (value);
-  model = gtk_combo_box_get_model (GTK_COMBO_BOX (user_data));
-  valid = gtk_tree_model_iter_nth_child (model, &iter, NULL, n);
-
-  if (valid) {
-    char *item_name;
-    gtk_tree_model_get (model, &iter, 1, &item_name, -1);
-
-    variant = g_variant_new_string (item_name);
-
-    g_free (item_name);
-  }
-
-  return variant;
-}
-
-static void
-create_node_combo (PrefsDialog   *dialog,
-                   EphyEncodings *encodings,
-                   const char    *default_value)
-{
-  GtkComboBox *combo;
-  GtkCellRenderer *renderer;
-  GtkListStore *store;
-  GList *all_encodings, *p;
-  char *code;
-
-  code = g_settings_get_string (EPHY_SETTINGS_WEB, EPHY_PREFS_WEB_DEFAULT_ENCODING);
-  if (code == NULL || ephy_encodings_get_encoding (encodings, code, FALSE) == NULL)
-  {
-    /* safe default */
-    g_settings_set_string (EPHY_SETTINGS_WEB,
-                           EPHY_PREFS_WEB_DEFAULT_ENCODING,
-                           default_value);
-  }
-  g_free (code);
-
-  combo = GTK_COMBO_BOX (dialog->default_encoding_combo);
-
-  store = gtk_list_store_new (NUM_COLS, G_TYPE_STRING, G_TYPE_STRING);
-  all_encodings = ephy_encodings_get_all (encodings);
-  for (p = all_encodings; p; p = p->next) {
-    GtkTreeIter iter;
-    EphyEncoding *encoding = EPHY_ENCODING (p->data);
-
-    gtk_list_store_append (store, &iter);
-    gtk_list_store_set (store, &iter,
-                        COL_TITLE_ELIDED,
-                        ephy_encoding_get_title_elided (encoding),
-                        -1);
-    gtk_list_store_set (store, &iter,
-                        COL_ENCODING,
-                        ephy_encoding_get_encoding (encoding),
-                        -1);
-  }
-  g_list_free (all_encodings);
-
-  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store), COL_TITLE_ELIDED,
-                                        GTK_SORT_ASCENDING);
-  gtk_combo_box_set_model (combo, GTK_TREE_MODEL (store));
-
-  renderer = gtk_cell_renderer_text_new ();
-  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
-  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), renderer,
-                                  "text", COL_TITLE_ELIDED,
-                                  NULL);
-
-  g_settings_bind_with_mapping (EPHY_SETTINGS_WEB,
-                                EPHY_PREFS_WEB_DEFAULT_ENCODING,
-                                combo, "active",
-                                G_SETTINGS_BIND_DEFAULT,
-                                combo_get_mapping,
-                                combo_set_mapping,
-                                combo,
-                                NULL);
-}
-
 static void
 language_editor_add (PrefsDialog *pd,
                      const char  *code,
@@ -1425,116 +1306,6 @@ cookies_set_mapping (const GValue       *value,
   return variant;
 }
 
-static void
-search_engine_combo_add_default_engines (GtkListStore *store)
-{
-  guint i;
-  const char *default_engines[][3] = {       /* Search engine option in the preferences dialog */
-    { N_("DuckDuckGo"),
-      "https://duckduckgo.com/?q=%s&t=epiphany",
-      /* For the preferences dialog. Must exactly match the URL
-      * you chose in the gschema, but with & instead of &amp;
-      * If the match is not exact, there will be a spurious, ugly
-      * entry in the preferences combo, so please test this. */
-      N_("https://duckduckgo.com/?q=%s&t=epiphany") },
-    /* Search engine option in the preferences dialog */
-    { N_("Google"),
-      "https://google.com/search?q=%s",
-      /* For the preferences dialog. Consider a regional variant, like google.co.uk */
-      N_("https://google.com/search?q=%s") },
-    /* Search engine option in the preferences dialog */
-    { N_("Bing"),
-      "https://www.bing.com/search?q=%s",
-      /* For the preferences dialog. Consider a regional variant, like uk.bing.com */
-      N_("https://www.bing.com/search?q=%s") }
-  };
-
-  for (i = 0; i < G_N_ELEMENTS (default_engines); ++i) {
-    gtk_list_store_insert_with_values (store, NULL, -1,
-                                       SEARCH_ENGINE_COL_NAME,
-                                       _(default_engines[i][0]),
-                                       SEARCH_ENGINE_COL_STOCK_URL,
-                                       default_engines[i][1],
-                                       SEARCH_ENGINE_COL_URL,
-                                       _(default_engines[i][2]),
-                                       -1);
-  }
-}
-
-/* Has the user manually set the engine to something not in the combo?
- * If so, add that URL as an extra item in the combo. */
-static void
-search_engine_combo_add_current_engine (GtkListStore *store)
-{
-  GtkTreeIter iter;
-  char *original_url;
-  gboolean in_combo = FALSE;
-  gboolean has_next = FALSE;
-
-  original_url = g_settings_get_string (EPHY_SETTINGS_MAIN,
-                                        EPHY_PREFS_KEYWORD_SEARCH_URL);
-  if (!original_url)
-    return;
-
-  has_next = gtk_tree_model_get_iter_first (GTK_TREE_MODEL (store), &iter);
-
-  while (!in_combo && has_next) {
-    char *stock_url, *url;
-
-    gtk_tree_model_get (GTK_TREE_MODEL (store), &iter,
-                        SEARCH_ENGINE_COL_STOCK_URL, &stock_url,
-                        SEARCH_ENGINE_COL_URL, &url, -1);
-
-    if (strcmp (original_url, stock_url) == 0 ||
-        strcmp (original_url, url) == 0)
-      in_combo = TRUE;
-
-    g_free (stock_url);
-    g_free (url);
-    has_next = gtk_tree_model_iter_next (GTK_TREE_MODEL (store), &iter);
-  }
-
-  if (!in_combo)
-    gtk_list_store_insert_with_values (store, NULL, -1,
-                                       SEARCH_ENGINE_COL_NAME, original_url,
-                                       SEARCH_ENGINE_COL_STOCK_URL, original_url,
-                                       SEARCH_ENGINE_COL_URL, original_url,
-                                       -1);
-
-  g_free (original_url);
-}
-
-static void
-create_search_engine_combo (GtkComboBox *combo)
-{
-  GtkCellRenderer *renderer;
-  GtkListStore *store;
-
-  store = GTK_LIST_STORE (gtk_combo_box_get_model (combo));
-
-  search_engine_combo_add_default_engines (store);
-  search_engine_combo_add_current_engine (store);
-
-  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store), SEARCH_ENGINE_COL_NAME,
-                                        GTK_SORT_ASCENDING);
-  gtk_combo_box_set_model (combo, GTK_TREE_MODEL (store));
-
-  renderer = gtk_cell_renderer_text_new ();
-  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
-  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), renderer,
-                                  "text", SEARCH_ENGINE_COL_NAME,
-                                  NULL);
-
-  g_settings_bind_with_mapping (EPHY_SETTINGS_MAIN,
-                                EPHY_PREFS_KEYWORD_SEARCH_URL,
-                                combo, "active",
-                                G_SETTINGS_BIND_DEFAULT,
-                                combo_get_mapping,
-                                combo_set_mapping,
-                                combo,
-                                NULL);
-}
-
 static gboolean
 new_tab_homepage_get_mapping (GValue   *value,
                               GVariant *variant,
@@ -1756,7 +1527,6 @@ setup_general_page (PrefsDialog *dialog)
                     dialog);
 
   create_download_path_button (dialog);
-  create_search_engine_combo (GTK_COMBO_BOX (dialog->search_engine_combo));
 }
 
 static void
@@ -1858,7 +1628,6 @@ static void
 setup_language_page (PrefsDialog *dialog)
 {
   GSettings *web_settings;
-  EphyEncodings *encodings;
 
   web_settings = ephy_settings_get (EPHY_PREFS_WEB_SCHEMA);
 
@@ -1868,10 +1637,6 @@ setup_language_page (PrefsDialog *dialog)
                    "active",
                    G_SETTINGS_BIND_DEFAULT);
 
-  encodings = EPHY_ENCODINGS (ephy_embed_shell_get_encodings (EPHY_EMBED_SHELL (ephy_shell_get_default ())));
-
-  create_node_combo (dialog, encodings, "ISO-8859-1");
-
   create_language_section (dialog);
 }
 
